NS_FERD_1p0 = "urn:ferd:CrossIndustryDocument:invoice:1p0"
NS_UDT = "urn:un:unece:uncefact:data:standard:UnqualifiedDataType:15"
NS_RAM = "urn:un:unece:uncefact:data:standard:ReusableAggregateBusinessInformationEntity:12"
BASIC = "BASIC"
COMFORT = "COMFORT"
EXTENDED = "EXTENDED"
